#!/usr/bin/env python3
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    """
    Returns a sample with cylinders on a substrate,
    forming a 2D lattice with different disorder rotated lattice
    """
    m_vacuum = ba.RefractiveMaterial("Vacuum", 0, 0)
    m_substrate = ba.RefractiveMaterial("Substrate", 6e-6, 2e-8)
    m_particle = ba.RefractiveMaterial("Particle", 6e-4, 2e-8)

    vacuum_layer = ba.Layer(m_vacuum)
    substrate_layer = ba.Layer(m_substrate)

    p_interparticle = \
        ba.Interference2DLattice(ba.SquareLattice2D(25*nm, 0))
    pdf = ba.Profile2DCauchy(48*nm, 16*nm, 0)
    p_interparticle.setDecayFunction(pdf)

    particle_layout = ba.ParticleLayout()
    ff = ba.Cylinder(3*nm, 3*nm)
    position = R3(0, 0, 0)
    cylinder = ba.Particle(m_particle, ff.clone())
    cylinder.translate(position)
    particle_layout.addParticle(cylinder)
    particle_layout.setInterference(p_interparticle)

    vacuum_layer.addLayout(particle_layout)

    sample = ba.MultiLayer()
    sample.addLayer(vacuum_layer)
    sample.addLayer(substrate_layer)
    return sample


def get_simulation(sample):
    beam = ba.UnitBeam(0.1*nm, 0.2*deg)
    detector = ba.SphericalDetector(bp.simargs['n'], 2*deg, 1*deg, 1*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    distr_1 = ba.DistributionGate(0, 240*deg)
    simulation.addParameterDistribution("*/SquareLattice2D/Xi", distr_1, 3,
                                        0)
    return simulation


if __name__ == '__main__':
    bp.parse_args(sim_n=100)
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    bp.plot_simulation_result(result)
